@@ -1,3 +1,23 @@
+(* 
+ * File - File manipulation
+ * Copyright (C) 2008 David Teller
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version,
+ * with the special exception on linking described in file LICENSE.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *)
+
 open BatIO
 open ListLabels
 open Unix
@@ -89,7 +109,7 @@ let out_chan_mode ?mode binary =
     | Some l -> Open_wronly :: (mode_to_open_flag l)
 
 
-let open_out ?mode ?(perm=user_read lor user_write) name =
+let open_out ?mode ?(perm=0o666) name =
 (*  Printf.eprintf "Opening out\n%!";*)
   output_channel ~cleanup:true (open_out_gen (out_chan_mode ?mode true) perm name)
 
