@@ -158,9 +158,9 @@ val iter : ('a -> 'b -> unit) -> ('a, 'b) t -> unit
 
 val fold : ('a -> 'b -> 'c -> 'c) -> ('a, 'b) t -> 'c -> 'c
 (** [Hashtbl.fold f tbl init] computes
-   [(f kN dN ... (f k1 d1 init)...)],
-   where [k1 ... kN] are the keys of all bindings in [tbl],
-   and [d1 ... dN] are the associated values.
+   [(f kN dN ... (f k1 d1 (f k0 d0 init))...)],
+   where [k0,k1..kN] are the keys of all bindings in [tbl],
+   and [d0,d1..dN] are the associated values.
    Each binding is presented exactly once to [f].
    The order in which the bindings are passed to [f] is unspecified.
    However, if the table contains several bindings for the same key,
@@ -184,8 +184,8 @@ val filteri: ('key -> 'a -> bool) -> ('key, 'a) t -> ('key, 'a) t
 
 val filter_map: ('key -> 'a -> 'b option) -> ('key, 'a) t -> ('key, 'b) t
   (** [filter_map f m] combines the features of [filteri] and
-      [map].  It calls calls [f key0 a0], [f key1 a1], [f keyn an]
-      where [a0..an] are the elements of [m] and [key0..keyn] the
+      [map].  It calls [f key0 a0], [f key1 a1], [f keyn an]
+      where [a0,a1..an] are the elements of [m] and [key0..keyn] the
       respective corresponding keys. It returns the map of
       pairs [keyi],[bi] such as [f keyi ai = Some bi] (when [f] returns
       [None], the corresponding element of [m] is discarded). *)
@@ -416,16 +416,11 @@ module Cap :
 sig
 
   type ('a, 'b, 'c) t constraint 'c = [< `Read | `Write ]
-      (** The type of a hashtable. *)
+  (** The type of a hashtable. *)
       
   (**{6 Constructors}*)
 
 val create : int -> ('a, 'b, _) t
-(** [Hashtbl.create n] creates a new, empty hash table, with
-   initial size [n].  For best results, [n] should be on the
-   order of the expected number of elements that will be in
-   the table.  The table grows as needed, so [n] is just an
-   initial guess. *)
 
 external of_table  : ('a, 'b) Hashtbl.t -> ('a, 'b, _ ) t = "%identity"
   (** Adopt a regular hashtable as a capability hashtble, allowing
@@ -455,65 +450,33 @@ external write_only : ('a, 'b, [>`Write]) t -> ('a, 'b, [`Write]) t = "%identity
 
 (**{6 Base operations}*)
 val length : ('a, 'b, _) t -> int
-(** [Hashtbl.length tbl] returns the number of bindings in [tbl].
-   Multiple bindings are counted multiply, so [Hashtbl.length]
-   gives the number of times [Hashtbl.iter] calls its first argument. *)
 
 val is_empty : ('a, 'b, _) t -> bool
-  (** [Hashtbl.is_empty tbl] returns [true] if there are no bindings
-      in [tbl], false otherwise.*)
 
 val add : ('a, 'b, [>`Write]) t -> 'a -> 'b -> unit
-(** [Hashtbl.add tbl x y] adds a binding of [x] to [y] in table [tbl].
-   Previous bindings for [x] are not removed, but simply
-   hidden. That is, after performing {!Hashtbl.remove}[ tbl x],
-   the previous binding for [x], if any, is restored.
-   (Same behavior as with association lists.) *)
 
 val remove : ('a, 'b, [>`Write]) t -> 'a -> unit
-(** [Hashtbl.remove tbl x] removes the current binding of [x] in [tbl],
-   restoring the previous binding if it exists.
-   It does nothing if [x] is not bound in [tbl]. *)
 
 val remove_all : ('a,'b, [>`Write]) t -> 'a -> unit
-  (** Remove all bindings for the given key *)
 
 val replace : ('a, 'b, [>`Write]) t -> 'a -> 'b -> unit
-(** [Hashtbl.replace tbl x y] replaces the current binding of [x]
-   in [tbl] by a binding of [x] to [y].  If [x] is unbound in [tbl],
-   a binding of [x] to [y] is added to [tbl].
-   This is functionally equivalent to {!Hashtbl.remove}[ tbl x]
-   followed by {!Hashtbl.add}[ tbl x y]. *)
 
 val copy : ('a, 'b, [>`Read]) t -> ('a, 'b, _) t
-(** Return a copy of the given hashtable. *)
 
 val clear : ('a, 'b, [>`Write]) t -> unit
-(** Empty a hash table. *)
 
 
 (**{6 Searching}*)
 
 val find : ('a, 'b, [>`Read]) t -> 'a -> 'b
-(** [Hashtbl.find tbl x] returns the current binding of [x] in [tbl],
-   or raises [Not_found] if no such binding exists. *)
 
 val find_all : ('a, 'b, [>`Read]) t -> 'a -> 'b list
-(** [Hashtbl.find_all tbl x] returns the list of all data
-   associated with [x] in [tbl].
-   The current binding is returned first, then the previous
-   bindings, in reverse order of introduction in the table. *)
 
 val find_default : ('a, 'b, [>`Read]) t -> 'a -> 'b -> 'b
-  (** Find a binding for the key, and return a default
-      value if not found *)
 
 val find_option : ('a, 'b, [>`Read]) t -> 'a -> 'b option
-  (** Find a binding for the key, or return [None] if no
-      value is found *)
 
 val mem : ('a, 'b, [>`Read]) t -> 'a -> bool
-(** [Hashtbl.mem tbl x] checks if [x] is bound in [tbl]. *)
 
 (*val exists : ('a,'b) t -> 'a -> bool*)
   (** [exists h k] returns true is at least one item with key [k] is
@@ -521,64 +484,26 @@ val mem : ('a, 'b, [>`Read]) t -> 'a -> bool
 
 (**{6 Traversing}*)
 val iter : ('a -> 'b -> unit) -> ('a, 'b, [>`Read]) t -> unit
-(** [Hashtbl.iter f tbl] applies [f] to all bindings in table [tbl].
-   [f] receives the key as first argument, and the associated value
-   as second argument. Each binding is presented exactly once to [f].
-   The order in which the bindings are passed to [f] is unspecified.
-   However, if the table contains several bindings for the same key,
-   they are passed to [f] in reverse order of introduction, that is,
-   the most recent binding is passed first. *)
 
 val fold : ('a -> 'b -> 'c -> 'c) -> ('a, 'b, [>`Read]) t -> 'c -> 'c
-(** [Hashtbl.fold f tbl init] computes
-   [(f kN dN ... (f k1 d1 init)...)],
-   where [k1 ... kN] are the keys of all bindings in [tbl],
-   and [d1 ... dN] are the associated values.
-   Each binding is presented exactly once to [f].
-   The order in which the bindings are passed to [f] is unspecified.
-   However, if the table contains several bindings for the same key,
-   they are passed to [f] in reverse order of introduction, that is,
-   the most recent binding is passed first. *)
 
 val map : ('a -> 'b -> 'c) -> ('a, 'b, [>`Read]) t -> ('a, 'c, _) t
-  (** [map f x] creates a new hashtable with the same
-      keys as [x], but with the function [f] applied to
-      all the values *)
 
 val filter: ('a -> bool) -> ('key, 'a, [>`Read]) t -> ('key, 'a, _) t
-  (**[filter f m] returns a new hashtable where only the values [a] of [m]
-     such that [f a = true] remain.*)
 
 val filteri: ('key -> 'a -> bool) -> ('key, 'a, [>`Read]) t -> ('key, 'a, _) t
-  (**[filter f m] returns a map where only the key, values pairs
-     [key], [a] of [m] such that [f key a = true] remain. The
-     bindings are passed to [f] in increasing order with respect
-     to the ordering over the type of the keys. *)
 
 val filter_map: ('key -> 'a -> 'b option) -> ('key, 'a, [>`Read]) t -> ('key, 'b, _) t
-  (** [filter_map f m] combines the features of [filteri] and
-      [map].  It calls calls [f key0 a0], [f key1 a1], [f keyn an]
-      where [a0..an] are the elements of [m] and [key0..keyn] the
-      respective corresponding keys. It returns the map of
-      pairs [keyi],[bi] such as [f keyi ai = Some bi] (when [f] returns
-      [None], the corresponding element of [m] is discarded). *)
-
 
 (**{6 Conversions}*)
 
 val keys : ('a,'b, [>`Read]) t -> 'a BatEnum.t
-  (** Return an enumeration of all the keys of a hashtable.
-      If the key is in the Hashtable multiple times, all occurrences
-      will be returned.  *)
 
 val values : ('a, 'b, [>`Read]) t -> 'b BatEnum.t
-  (** Return an enumeration of all the values of a hashtable. *)
 
 val enum : ('a, 'b, [>`Read]) t -> ('a * 'b) BatEnum.t
-  (** Return an enumeration of (key,value) pairs of a hashtable. *)
 
 val of_enum : ('a * 'b) BatEnum.t -> ('a, 'b, _) t
-  (** Create a hashtable from a (key,value) enumeration. *)
 
 (** {6 Boilerplate code}*)
 
@@ -591,27 +516,14 @@ val print :  ?first:string -> ?last:string -> ?sep:string -> ('a BatInnerIO.outp
 
 
 (** {6 Override modules}*)
-  
-(**
-   The following modules replace functions defined in {!Hashtbl} with functions
-   behaving slightly differently but having the same name. This is by design:
-   the functions meant to override the corresponding functions of {!Hashtbl}.
-   *)
-  
-(** Operations on {!Hashtbl} without exceptions.*)
+    
+(** Operations on {!BatHashtbl.Cap} without exceptions.*)
 module Exceptionless :
 sig
   val find : ('a, 'b, [>`Read]) t -> 'a -> 'b option
 end
   
-(** Operations on {!Hashtbl} with labels.
-    
-    This module overrides a number of functions of {!Hashtbl} by
-    functions in which some arguments require labels. These labels are
-    there to improve readability and safety and to let you change the
-    order of arguments to functions. In every case, the behavior of the
-    function is identical to that of the corresponding function of {!Hashtbl}.
-*)
+(** Operations on {!BatHashtbl.Cap} with labels.*)
 module Labels :
 sig
   val add : ('a, 'b, [>`Write]) t -> key:'a -> data:'b -> unit
